#!/usr/bin/env python3
#from itertools import combinations
from collections import namedtuple
from collections import defaultdict
import csv
import gzip
import sys
import argparse
import operator
import random
import time
import os

class HumanRankingsData(object):
    def __init__(self):
        self.pair_wise_comparisons = defaultdict(list)
        pass

    def load_data(self, file_like):
        with gzip.open(file_like, mode="rt") as fl:
            for line in csv.DictReader(fl):
                direction = line['system1Id'].rsplit('.' , 2)[1]

                SystemsTuple = namedtuple("SystemTuple", "id rank")

                systems_ranks1 = []
                systems_ranks2 = []

                system1Id = line['system1Id'].rsplit('+')
                system2Id = line['system2Id'].rsplit('+')

                for sid in system1Id:
                    systems_ranks1.append(SystemsTuple(id = sid.rsplit('.', 2)[0], rank = int(line['system1rank'])))

                for sid in system2Id:
                    systems_ranks2.append(SystemsTuple(id = sid.rsplit('.', 2)[0], rank = int(line['system2rank'])))

                # Extract all tuples of systems, where first system was ranked better than second one
                extracted_tuples = [
                        (system1.id, system2.id)
                        for system1 in systems_ranks1
                        for system2 in systems_ranks2
                        if system1.rank < system2.rank
                        and system1.rank != -1
                        and system2.rank != -1
                    ]

                self.pair_wise_comparisons[direction] += extracted_tuples

    def human_scores(self, direction):
        comparisons = self.pair_wise_comparisons[direction]

        if config.bootstrap != 0:
            sample = [random.choice(comparisons) for _ in comparisons]
            comparisons = sample

        totals = defaultdict(int)
        betters = defaultdict(int)
        for better_system, worse_system in comparisons:
            totals[better_system] += 1
            totals[worse_system] += 1
            betters[better_system] += 1

        return { system : float(betters[system]) / float(totals[system]) for system in totals }

    def directions(self):
        return self.pair_wise_comparisons.keys()

            

def main():
    parser = argparse.ArgumentParser(
            description="""This script takes file with all human judgments collected during
            the wmt manual evaluation and computes the official human scores for each system
            in each language direction.
            """)

    parser.add_argument("judgments_file",
            help="file with human judgments, if ommited or '-' used stdin will be used.",
            metavar="FILE",
            nargs='?',
            #type=argparse.FileType('r'),
            default=sys.stdin)

    parser.add_argument("--tabulate",
            action='store_true',
            help="Print tables similar to those in wmt overview paper.")

    parser.add_argument("--bootstrap",
            help="Performs the bootstrap resampling and computes 0.95 confidence"
                 " intervals. The optional parameter specifies the number of new"
                 " randomly sampled sets of human comparisons",
            metavar="N",
            nargs='?',
            const=1000,
            default=0,
            type=int,
            )

    parser.add_argument("--rseed",
            help="Random seed used to generate samples when bootstrapping (default is unix timestamp)",
            metavar="N",
            default=int(time.time()),
            type=int,
            )

    parser.add_argument("--outputfolder",
            help="folder where to store bootstrap output",
            )

    global config
    config = parser.parse_args()
    
    data = HumanRankingsData()
    data.load_data(config.judgments_file)

    if config.bootstrap != 0:
        random.seed(config.rseed)
        if not os.path.exists(config.outputfolder):
            os.makedirs(config.outputfolder)
        for i in range(config.bootstrap):
            out = open(config.outputfolder + '/' + str(i) + '.human.scores', 'w')
            for direction in data.directions():
                for system, score in data.human_scores(direction).items():
                    spl = system.split('.', 1)
                    out.write("\t".join(["human", direction, spl[0], spl[1], str(score)]) + "\n")
                    out.flush()
            out.close()
        return

    if config.tabulate:
        for direction in data.directions():
            scores = data.human_scores(direction)
            print(direction)
            for system, score in sorted(scores.items(), key=operator.itemgetter(1), reverse=True):
                print("\t".join([system, str(score)]))
            print("")
    else:
        for direction in data.directions():
            for system, score in data.human_scores(direction).items():
                spl = system.split('.', 1)
                print("\t".join(["human", direction, spl[0], spl[1], str(score)]))

            
if __name__ == "__main__":
    main()

